#ifndef TEXTURE_H
#define TEXTURE_H

#include <stdio.h>


#if defined _WIN32
	#include <windows.h>
#else
	#include <stdlib.h>

	
#include <GL/glut.h>	
	
	typedef struct
	{
		unsigned int   biSize;
		int            biWidth;
		int            biHeight;
		unsigned short biPlanes;
		unsigned short biBitCount;
		unsigned int   biCompression;
		unsigned int   biSizeImage;
		int            biXPelsPerMeter;
		int            biYPelsPerMeter;
		unsigned int   biClrUsed;
		unsigned int   biClrImportant;
		char *data;
	} BITMAPINFOHEADER;
#endif

class Texture
{
public:
	GLuint tex[3];
public:
	Texture(char* file)
	{
		this->LoadBitmap(file);
	}

	~Texture() { }

#if defined _WIN32
	void LoadBitmap(char* filename)
	{
		int i, j=0; //Index variables
		FILE *l_file; //File pointer
		unsigned char *l_texture; //The pointer to the memory zone in which we will load the texture
	     
		// windows.h gives us these types to work with the Bitmap files
		BITMAPFILEHEADER fileheader; 
		BITMAPINFOHEADER infoheader;
		RGBTRIPLE rgb;

		if( (l_file = fopen(filename, "rb"))==NULL)
		{
			return;
		}
	    
		fread(&fileheader, sizeof(fileheader), 1, l_file); // Read the fileheader
	    
		fseek(l_file, sizeof(fileheader), SEEK_SET); // Jump the fileheader
		fread(&infoheader, sizeof(infoheader), 1, l_file); // and read the infoheader

		// Now we need to allocate the memory for our image (width * height * color deep)
		l_texture = (byte *) malloc(infoheader.biWidth * infoheader.biHeight * 3);
		// And fill it with zeros
		memset(l_texture, 0, infoheader.biWidth * infoheader.biHeight * 3);
	 
		// At this point we can read every pixel of the image
		for (i=0; i < infoheader.biWidth*infoheader.biHeight; i++)
		{            
				// We load an RGB value from the file
				fread(&rgb, sizeof(rgb), 1, l_file); 

				// And store it
				l_texture[j+0] = rgb.rgbtRed; // Red component
				l_texture[j+1] = rgb.rgbtGreen; // Green component
				l_texture[j+2] = rgb.rgbtBlue; // Blue component
				j += 3;
		}

		fclose(l_file); // Closes the file stream
	    
		glGenTextures(3, &tex[0]);

		// Create Nearest Filtered Texture
		glBindTexture(GL_TEXTURE_2D, tex[0]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, infoheader.biWidth, infoheader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, l_texture);

		// Create Linear Filtered Texture
		glBindTexture(GL_TEXTURE_2D, tex[1]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, infoheader.biWidth, infoheader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, l_texture);

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, tex[2]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, infoheader.biWidth, infoheader.biHeight, GL_RGB, GL_UNSIGNED_BYTE, l_texture);

		free(l_texture); // Free the memory we used to load the texture
	}
#else
	void LoadBitmap(char *filename)
	{
		FILE * file;
		char temp;
		long i;

		BITMAPINFOHEADER infoheader;

		if( (file = fopen(filename, "rb"))==NULL) // Open the file for reading
		{
			return;
		}

		fseek(file, 18, SEEK_CUR);  /* start reading width & height */
		fread(&infoheader.biWidth, sizeof(int), 1, file);

		fread(&infoheader.biHeight, sizeof(int), 1, file);

		fread(&infoheader.biPlanes, sizeof(short int), 1, file);
		if (infoheader.biPlanes != 1) {
			printf("Planes from %s is not 1: %u\n", filename, infoheader.biPlanes);
			return;
		}

		// read the bpp
		fread(&infoheader.biBitCount, sizeof(unsigned short int), 1, file);
		if (infoheader.biBitCount != 24) {
			printf("Bpp from %s is not 24: %d\n", filename, infoheader.biBitCount);
			return;
		}

		fseek(file, 24, SEEK_CUR);

		// read the data.
		infoheader.data = (char *) malloc(infoheader.biWidth * infoheader.biHeight * 3);
		if (infoheader.data == NULL) {
			printf("Error allocating memory for color-corrected image data\n");
			return;
		}

		if ((i = fread(infoheader.data, infoheader.biWidth * infoheader.biHeight * 3, 1, file)) != 1) {
			printf("Error reading image data from %s.\n", filename);
			return;
		}

		for (i=0; i<(infoheader.biWidth * infoheader.biHeight * 3); i+=3) { // reverse all of the colors. (bgr -> rgb)
			temp = infoheader.data[i];
			infoheader.data[i] = infoheader.data[i+2];
			infoheader.data[i+2] = temp;
		}

		fclose(file); // Closes the file stream

		glGenTextures(3, &tex[0]);

		// Create Nearest Filtered Texture
		glBindTexture(GL_TEXTURE_2D, tex[0]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, infoheader.biWidth, infoheader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, infoheader.data);

		// Create Linear Filtered Texture
		glBindTexture(GL_TEXTURE_2D, tex[1]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, infoheader.biWidth, infoheader.biHeight, 0, GL_RGB, GL_UNSIGNED_BYTE, infoheader.data);

		// Create MipMapped Texture
		glBindTexture(GL_TEXTURE_2D, tex[2]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, infoheader.biWidth, infoheader.biHeight, GL_RGB, GL_UNSIGNED_BYTE, infoheader.data);

		free(infoheader.data); // Free the memory we used to load the texture
	}
#endif
};

#endif
